#!/usr/bin/env python3
"""
SPS Search for amongus
by Javantea
July 27, 2022

Find seeds with the most number of I pieces in the first 300 pieces.
This is very slow, so use pypy3 or run multithreaded.
3m 36s on pypy3 with 32 threads.
"""
import tetris_sps
import multiprocessing

def compare(x, y):
    for i in range(min(len(x), len(y))):
        if x[i] != y[i]: return i
    return min(len(x), len(y)) + 1

def getSequenceI(args):
    piece_name = 'I'
    seed, num_pieces = args
    sps = tetris_sps.ThreadSafeSPS(seed)
    v = [tetris_sps.lookup[sps.pickTetriminoSeed()] for i in range(num_pieces)]
    res = compare(v, list('OLITZLJTSZ'))
    return (seed, res)

def main():
    import sys
    import argparse
    parser = argparse.ArgumentParser(description="TetrisGYM amongus search")
    parser.add_argument('-o', '--output', action='store_true')
    #parser.add_argument('start', default=0, type=int)
    parser.add_argument('above', default=0, type=int, nargs='?')
    args = parser.parse_args()
    #start = args.start
    above = args.above
    max_i = 0
    num_pieces = 10
    pool = multiprocessing.Pool()
    res = pool.imap(getSequenceI, [(x, num_pieces) for x in range(0x1000000)], chunksize=1000)
    if args.output:
        fout = open('tetris_sps.dat', 'w')
    for seed, num_i in res:
        if args.output:
            # We just want the output so we can graph sorted
            fout.write('{0}\n'.format(num_i))
        if num_i > max_i or (above > 0 and num_i >= above):
            if num_i > max_i:
                best_seed = seed
                max_i = num_i
            print('{0:06x} {1} amongus'.format(seed, num_i))
    if args.output:
        fout.close()

if __name__ == '__main__':
    main()
